/* Defines and configuration for the CEC (HDMI Consumer Electronic Control) Library
  Copyright (c) 2009 Michael Dreher <michael(at)5dot1.de>  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//#pragma once

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <WProgram.h>
//#include <HardwareSerial.h>

#define CECDEBUG 1

// pin number defined by Arduino BT board
#define RESET static_cast<byte>(7) // BT module uses pin 7 on PORT D for reset

#define LED static_cast<byte>(5)  // select the pin for the LED
#define LEDPORT PORTB // select the port for the LED
#define LEDPIN PINB
#define LEDDDR DDRB

// these are bit numbers of PORTD and not logical pin numbers
#define CEC_IN_BIT_NR 2 // bit number / PCINT
#define CEC_IN_PORT_NAME D
#define CEC_IN_PORT_NO 2 // number for IRQ related registers, PORTB => 0, PORTC => 1, PORTD => 2
//#define CEC_IN_PIN_NR (CEC_IN_BIT_NR + (8 * (CEC_IN_PORT_NO == 2))) // pin number on Arduino board

#define CEC_OUT_BIT_NR 3 // bit number / PCINT
#define CEC_OUT_PORT_NAME D
#define CEC_OUT_PORT_NO 2 // number for IRQ related registers, PORTB => 0, PORTC => 1, PORTD => 2
//#define CEC_OUT_PIN_NR (CEC_OUT_BIT_NR + (8 * (CEC_OUT_PORT_NO == 2))) // pin number on Arduino board

#define CEC_TIMEOUT 5000
#define CEC_SILENTTIME 10

//--------------------------------------------------------------------------------------------------------

#define CEC_STATE_WAITFORHOTPLUG static_cast<byte>(0)
#define CEC_STATE_IDLE static_cast<byte>(1)
#define CEC_STATE_RECEIVE static_cast<byte>(2)
#define CEC_STATE_RECEIVED static_cast<byte>(3)
#define CEC_STATE_SEND static_cast<byte>(4)
#define CEC_STATE_ERROR static_cast<byte>(5)

#define CEC_STATE_RECEIVE_WAITFORSTARTBITTIMEA static_cast<byte>(0)
#define CEC_STATE_RECEIVE_WAITFORSTARTBITTIMEB static_cast<byte>(1)
#define CEC_STATE_RECEIVE_WAITFORLO static_cast<byte>(2)
#define CEC_STATE_RECEIVE_WAITFORHI static_cast<byte>(3)
#define CEC_STATE_RECEIVE_SENDACK_WAITFORLOW static_cast<byte>(4)
#define CEC_STATE_RECEIVE_SENDACK_WAITFORTIME static_cast<byte>(5)

//--------------------------------------------------------------------------------------------------------

#define CEC_OUT_PINx2(x) PIN##x
#define CEC_OUT_PINx(x) CEC_OUT_PINx2(x)
#define CEC_OUT_PIN CEC_OUT_PINx(CEC_OUT_PORT_NAME)

#define CEC_OUT_PORTx2(x) PORT##x
#define CEC_OUT_PORTx(x) CEC_OUT_PORTx2(x)
#define CEC_OUT_PORT CEC_OUT_PORTx(CEC_OUT_PORT_NAME)

#define CEC_OUT_DDRx2(x) DDR##x
#define CEC_OUT_DDRx(x) CEC_OUT_DDRx2(x)
#define CEC_OUT_DDR CEC_OUT_DDRx(CEC_OUT_PORT_NAME)

//--------------------------------------------------------------------------------------------------------

#define CEC_IN_PINx2(x) PIN##x
#define CEC_IN_PINx(x) CEC_IN_PINx2(x)
#define CEC_IN_PIN CEC_IN_PINx(CEC_IN_PORT_NAME)

#define CEC_IN_PORTx2(x) PORT##x
#define CEC_IN_PORTx(x) CEC_IN_PORTx2(x)
#define CEC_IN_PORT CEC_IN_PORTx(CEC_IN_PORT_NAME)

#define CEC_IN_DDRx2(x) DDR##x
#define CEC_IN_DDRx(x) CEC_IN_DDRx2(x)
#define CEC_IN_DDR CEC_IN_DDRx(CEC_IN_PORT_NAME)

#define DATA_INT_VECT_NAMEx2(x) PCINT##x##_vect
#define DATA_INT_VECT_NAMEx(x) DATA_INT_VECT_NAMEx2(x)
#define DATA_INT_VECT_NAME DATA_INT_VECT_NAMEx(CEC_IN_PORT_NO)

#define DATA_PCMSKx2(x) PCMSK##x // PCMSK used by DATA, depends on the port
#define DATA_PCMSKx(x) DATA_PCMSKx2(x) // PCMSK used by DATA, depends on the port
#define DATA_PCMSK DATA_PCMSKx(CEC_IN_PORT_NO) // PCMSK used by DATA, depends on the port

#define DATA_PCIEx2(x) PCIE##x // PCIE bit used by DATA, depends on the port
#define DATA_PCIEx(x) DATA_PCIEx2(x) // PCIE bit used by DATA, depends on the port
#define DATA_PCIE DATA_PCIEx(CEC_IN_PORT_NO) // PCIE bit used by DATA, depends on the port

#define DATA_EICRA_ISCx1x2(x) ISC##x##1
#define DATA_EICRA_ISCx1x(x) DATA_EICRA_ISCx1x2(x)
#define DATA_EICRA_ISCx1 DATA_EICRA_ISCx1x(DATA_INT_NO)

#define DATA_EICRA_ISCx0x2(x) ISC##x##0
#define DATA_EICRA_ISCx0x(x) DATA_EICRA_ISCx0x2(x)
#define DATA_EICRA_ISCx0 DATA_EICRA_ISCx0x(DATA_INT_NO)

#define DATA_INTx2(x) INT##x
#define DATA_INTx(x) DATA_INTx2(x)
#define DATA_INT DATA_INTx(DATA_INT_NO)

#define DATA_INT_VECT_NAMEBx2(x) INT##x##_vect
#define DATA_INT_VECT_NAMEBx(x) DATA_INT_VECT_NAMEx2(x)
#define DATA_INT_VECT_NAMEB DATA_INT_VECT_NAMEx(CEC_IN_PORT_NO)

//--------------------------------------------------------------------------------------------------------

// clearBit and setBit
#ifndef cbi
#define cbi(sfr, bit)     (_SFR_BYTE(sfr) &= ~_BV(bit)) 
#endif
#ifndef sbi
#define sbi(sfr, bit)     (_SFR_BYTE(sfr) |= _BV(bit))  
#endif

#define CASTBYTE(x) static_cast<unsigned char>(x)

//--------------------------------------------------------------------------------------------------------

int readSerialResponse(void);
void CEC_init(void);
void startIrqCapture(void);
unsigned char checkCaptureFlags(void);

extern volatile byte CECOwnAddress;
extern volatile byte CECmsg[16];
extern volatile byte CECmsgLen;
extern volatile unsigned short CECBitCount;

#if CECDEBUG == 1
extern volatile byte CECErrNo;
extern volatile unsigned long CECErrInfo;
#endif


#define LEDon sbi(LEDPORT, LED)
#define LEDoff cbi(LEDPORT, LED)
inline void setLED(bool v)
{
  if(v)
    LEDon;
  else
    LEDoff;
}

inline void toggleLED(void)
{
  sbi(PINB, LED); 
}

inline void debug_P(const char *addr)
{
  uint8_t c;

  while ((c = pgm_read_byte(addr++)))
  {
    Serial.print(c);
  }
}

inline void debugln_P(const char *addr)
{
  debug_P(addr);
  Serial.println();
}

